@@ -6,21 +6,16 @@
   * Copyright 1997 Bernd Schmidt
   */
 
-#define A_FIBF_SCRIPT  (1<<6)
-#define A_FIBF_PURE    (1<<5)
-#define A_FIBF_ARCHIVE (1<<4)
-#define A_FIBF_READ    (1<<3)
-#define A_FIBF_WRITE   (1<<2)
-#define A_FIBF_EXECUTE (1<<1)
-#define A_FIBF_DELETE  (1<<0)
-
 struct hardfiledata {
     unsigned long size;
     int nrcyls;
     int secspertrack;
     int surfaces;
     int reservedblocks;
+    int blocksize;
     FILE *fd;
 };
 
+struct uaedev_mount_info;
+
 extern struct hardfiledata *get_hardfile_data (int nr);
