@@ -6,20 +6,18 @@
   * Copyright 1996 Bernd Schmidt
   */
 
-extern void do_start_program(void);
-extern void do_leave_program(void);
-extern void start_program(void);
-extern void leave_program(void);
-extern void real_main(int, char **);
-extern void usage(void);
-extern void parse_cmdline(int argc, char **argv);
+extern void do_start_program (void);
+extern void do_leave_program (void);
+extern void start_program (void);
+extern void leave_program (void);
+extern void real_main (int, char **);
+extern void usage (void);
+extern void parse_cmdline (int argc, char **argv);
 
-/* Contains the filename of .uaerc */
-extern char optionsfile[];
-extern void save_options(FILE *);
 
-extern void uae_reset(void);
-extern void uae_quit(void);
+extern void uae_reset (void);
+extern void uae_quit (void);
+extern void reset_all_systems (void);
 
 extern int quit_program;
 
