@@ -17,37 +17,52 @@ static void sigchldhandler(int foo)
 {
 }
 
-int gui_init(void)
+int gui_init (void)
 {
     return 0;
 }
 
-int gui_update(void)
+int gui_update (void)
 {
     return 0;
 }
 
-void gui_exit(void)
+void gui_exit (void)
 {
 }
 
-void gui_led(int led, int on)
+void gui_fps (int x)
 {
 }
 
-void gui_filename(int num, const char *name)
+void gui_led (int led, int on)
 {
 }
 
-static void getline(char *p)
+void gui_filename (int num, const char *name)
 {
 }
 
-void gui_handle_events(void)
+static void getline (char *p)
 {
 }
 
-void gui_changesettings(void)
+void gui_handle_events (void)
 {
 }
 
+void gui_changesettings (void)
+{
+}
+
+void gui_update_gfx (void)
+{
+}
+
+void gui_lock (void)
+{
+}
+
+void gui_unlock (void)
+{
+}
