@@ -6,7 +6,7 @@
   * (c) 1997 Gustavo Goedert
   */
 
-void HandleDisk(int num);
+void HandleOptions(void);
 void SaveScreen(void);
 void EnterDebug(void);
 void LeaveDebug(void);
